@@ -45,6 +45,10 @@
 #include <sys/stat.h>
 #include <sys/time.h>
 
+#ifdef LIBUUID
+#include <uuid/uuid.h>
+#endif
+
 #include "nvme-print.h"
 #include "nvme-ioctl.h"
 #include "nvme-lightnvm.h"
@@ -153,13 +157,7 @@ int parse_and_open(int argc, char **argv, const char *desc,
 
 static const char *output_format = "Output format: normal|json|binary";
 
-enum {
-	NORMAL,
-	JSON,
-	BINARY,
-};
-
-static int validate_output_format(char *format)
+int validate_output_format(char *format)
 {
 	if (!format)
 		return -EINVAL;
@@ -788,7 +786,7 @@ static const char *dev = "/dev/";
 /* Assume every block device starting with /dev/nvme is an nvme namespace */
 static int scan_dev_filter(const struct dirent *d)
 {
-	char path[256];
+	char path[264];
 	struct stat bd;
 	int ctrl, ns, part;
 
@@ -810,7 +808,7 @@ static int scan_dev_filter(const struct dirent *d)
 
 static int list(int argc, char **argv, struct command *cmd, struct plugin *plugin)
 {
-	char path[256];
+	char path[264];
 	struct dirent **devices;
 	struct list_item *list_items;
 	unsigned int i, n, fd, ret;
@@ -836,7 +834,7 @@ static int list(int argc, char **argv, struct command *cmd, struct plugin *plugi
 		return -EINVAL;
 
 	n = scandir(dev, &devices, scan_dev_filter, alphasort);
-	if (n <= 0)
+	if (n < 0)
 		return n;
 
 	list_items = calloc(n, sizeof(*list_items));
@@ -879,7 +877,7 @@ static int get_nsid(int fd)
 	return nsid;
 }
 
-int __id_ctrl(int argc, char **argv, struct command *cmd, struct plugin *plugin, void (*vs)(__u8 *vs))
+int __id_ctrl(int argc, char **argv, struct command *cmd, struct plugin *plugin, void (*vs)(__u8 *vs, struct json_object *root))
 {
 	const char *desc = "Send an Identify Controller command to "\
 		"the given device and report information about the specified "\
@@ -932,7 +930,7 @@ int __id_ctrl(int argc, char **argv, struct command *cmd, struct plugin *plugin,
 		if (fmt == BINARY)
 			d_raw((unsigned char *)&ctrl, sizeof(ctrl));
 		else if (fmt == JSON)
-			json_nvme_id_ctrl(&ctrl, flags);
+			json_nvme_id_ctrl(&ctrl, flags, vs);
 		else {
 			printf("NVME Identify Controller:\n");
 			__show_nvme_id_ctrl(&ctrl, flags, vs);
@@ -1030,7 +1028,9 @@ static int get_ns_id(int argc, char **argv, struct command *cmd, struct plugin *
 {
 	int nsid, fd;
 
-	fd = open_dev(argv[1]);
+	fd = get_dev(argc, argv);
+	if (fd < 0)
+		return fd;
 	nsid = nvme_get_nsid(fd);
 	if (nsid <= 0) {
 		perror(devicename);
@@ -1118,6 +1118,9 @@ static int get_feature(int argc, char **argv, struct command *cmd, struct plugin
 		cfg.data_len = 8;
 		break;
 	}
+
+	if (cfg.sel == 3)
+		cfg.data_len = 0;
 	
 	if (cfg.data_len) {
 		if (posix_memalign(&buf, getpagesize(), cfg.data_len)) {
@@ -1134,7 +1137,7 @@ static int get_feature(int argc, char **argv, struct command *cmd, struct plugin
 			printf("get-feature:%#02x (%s), %s value:%#08x\n", cfg.feature_id,
 				nvme_feature_to_string(cfg.feature_id),
 				nvme_select_to_string(cfg.sel), result);
-			if (cfg.human_readable)
+			if (cfg.human_readable && buf)
 				nvme_feature_show_fields(cfg.feature_id, result, buf);
 			else if (buf)
 				d(buf, cfg.data_len, 16, 1);
@@ -1394,7 +1397,9 @@ static int format(int argc, char **argv, struct command *cmd, struct plugin *plu
 	const char *ms = "[0-1]: extended format off/on";
 	const char *reset = "Automatically reset the controller after successful format";
 	const char *timeout = "timeout value, in milliseconds";
+	struct nvme_id_ns ns;
 	int err, fd;
+	__u8 prev_lbaf = 0;
 
 	struct config {
 		__u32 namespace_id;
@@ -1410,7 +1415,7 @@ static int format(int argc, char **argv, struct command *cmd, struct plugin *plu
 	struct config cfg = {
 		.namespace_id = 0xffffffff,
 		.timeout      = 120000,
-		.lbaf         = 0,
+		.lbaf         = 0xff,
 		.ses          = 0,
 		.pi           = 0,
 		.reset        = 0,
@@ -1432,6 +1437,24 @@ static int format(int argc, char **argv, struct command *cmd, struct plugin *plu
 	if (fd < 0)
 		return fd;
 
+	if (S_ISBLK(nvme_stat.st_mode))
+		cfg.namespace_id = get_nsid(fd);
+	if (cfg.namespace_id != 0xffffffff) {
+		err = nvme_identify_ns(fd, cfg.namespace_id, 0, &ns);
+		if (err) {
+			if (err < 0)
+				perror("identify-namespace");
+			else
+				fprintf(stderr,
+					"NVME Admin command error:%s(%x)\n",
+					nvme_status_to_string(err), err);
+			return err;
+		}
+		prev_lbaf = ns.flbas & 0xf;
+	}
+	if (cfg.lbaf == 0xff)
+		cfg.lbaf = prev_lbaf;
+
 	/* ses & pi checks set to 7 for forward-compatibility */
 	if (cfg.ses > 7) {
 		fprintf(stderr, "invalid secure erase settings:%d\n", cfg.ses);
@@ -1453,8 +1476,6 @@ static int format(int argc, char **argv, struct command *cmd, struct plugin *plu
 		fprintf(stderr, "invalid ms:%d\n", cfg.ms);
 		return EINVAL;
 	}
-	if (S_ISBLK(nvme_stat.st_mode))
-		cfg.namespace_id = get_nsid(fd);
 
 	err = nvme_format(fd, cfg.namespace_id, cfg.lbaf, cfg.ses, cfg.pi,
 				cfg.pil, cfg.ms, cfg.timeout);
@@ -1644,6 +1665,12 @@ static int sec_send(int argc, char **argv, struct command *cmd, struct plugin *p
 		return ENOMEM;
 	}
 
+	if (read(sec_fd, sec_buf, sec_size) < 0) {
+		fprintf(stderr, "Failed to read data from security file with %s\n",
+			strerror(errno));
+		return EINVAL;
+	}
+
 	err = nvme_sec_send(fd, cfg.namespace_id, cfg.nssf, cfg.spsp, cfg.secp,
 			cfg.tl, sec_size, sec_buf, &result);
 	if (err < 0)
@@ -2173,6 +2200,7 @@ static int resv_report(int argc, char **argv, struct command *cmd, struct plugin
 			show_nvme_resv_report(status);
 		}
 	}
+	free(status);
 	return err;
 }
 
@@ -2278,7 +2306,7 @@ static int submit_io(int opcode, char *command, const char *desc,
 	if (strlen(cfg.metadata)){
 		mfd = open(cfg.metadata, flags, mode);
 		if (mfd < 0) {
-			perror(cfg.data);
+			perror(cfg.metadata);
 			return EINVAL;
 		}
 	}
@@ -2470,7 +2498,7 @@ static int sec_recv(int argc, char **argv, struct command *cmd, struct plugin *p
 							result);
 			d(sec_buf, cfg.size, 16, 1);
 		} else if (cfg.size)
-			d_raw((unsigned char *)&sec_buf, cfg.size);
+			d_raw((unsigned char *)sec_buf, cfg.size);
 	}
 	return err;
 }
@@ -2671,6 +2699,26 @@ static int admin_passthru(int argc, char **argv, struct command *cmd, struct plu
 	return passthru(argc, argv, NVME_IOCTL_ADMIN_CMD, desc, cmd);
 }
 
+#ifdef LIBUUID
+static int gen_hostnqn_cmd(int argc, char **argv, struct command *command, struct plugin *plugin)
+{
+	uuid_t uuid;
+	char uuid_str[37]; /* e.g. 1b4e28ba-2fa1-11d2-883f-0016d3cca427 + \0 */
+
+	uuid_generate_random(uuid);
+	uuid_unparse_lower(uuid, uuid_str);
+	printf("nqn.2014-08.org.nvmexpress:NVMf:uuid:%s\n", uuid_str);
+	return 0;
+}
+#else
+static int gen_hostnqn_cmd(int argc, char **argv, struct command *command, struct plugin *plugin)
+{
+	fprintf(stderr, "\"%s\" not supported. Install lib uuid and rebuild.\n",
+		command->name);
+	return -ENOTSUP;
+}
+#endif
+
 static int discover_cmd(int argc, char **argv, struct command *command, struct plugin *plugin)
 {
 	const char *desc = "Send Get Log Page request to Discovery Controller.";
